/*
 * This technoligy demo tries to demonstrate the integration of SDL OpenGL and OpenAL.
 * All code snippets are copyright by the original authors of the SDL OpenGL and OpenAL demo's
 * Everything else is public domain
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef _W32
#include <windows.h>
#endif
#ifdef _UNIX
#include <X11/Xlib.h>
#include <GL/glx.h>
#endif

#include <AL/al.h>
#include <AL/alc.h>
#include <AL/alut.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL/SDL.h>

static GLfloat spin = 0.0;
static GLfloat spininc = 0.0;

static ALuint left_sid = 0;
static ALuint right_sid = 0;
static ALfloat left_pos[3] = { -4.0, 0.0, 4.0 };
static ALfloat right_pos[3] = { 4.0, 0.0, 4.0 };

void quit(int code)
{
#ifdef _W32
   ALvoid *context;
   ALCdevice *device;
#endif
/*
 * Quit SDL so we can release the fullscreen
 * mode and restore the previous video settings,
 * etc.
 */
     
   printf("Quiting SDL.\n");
#ifdef _WIN32
   context=alcGetCurrentContext();
   device=alcGetContextsDevice(context);
   alcDestroyContext(context);
   alcCloseDevice(device);
#else
   alutExit();
#endif
   SDL_Quit();

   exit(code);
}

void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT);
   glPushMatrix();
   glRotatef(spin, 0.0, 0.0, 1.0);
   glColor3f(1.0, 1.0, 1.0);
   glRectf(-25.0, -25.0, 25.0, 25.0);
   glPopMatrix();
   
   SDL_GL_SwapBuffers();
}

void reshape(int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-50.0, 50.0, -50.0, 50.0, -1.0, 1.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

void spinDisplay(void)
{
   spin = spin + spininc;
   if (spin > 360.0) {
      spin = spin - 360.0;
   }
}

void init(void) 
{
   ALfloat zeroes[] = {0.0f, 0.0f, 0.0f};
   ALfloat front[] = {0.0f, 0.0f, -1.0f, 0.0f, 1.0f, 0.0f};
   ALfloat back[] = {0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 0.0f};
   ALuint sources[2];
   ALuint boom;
   void* wave;
   ALsizei size;
   ALsizei freq;
   ALsizei format;
#ifdef _LINUX
   ALsizei bits;
#endif
#ifdef _W32
   ALboolean loop;
#endif

   alListenerfv(AL_POSITION, zeroes);
   alListenerfv(AL_VELOCITY, zeroes);
   alListenerfv(AL_ORIENTATION, front);
   
   glClearColor(0.0, 0.0, 0.0, 0.0);
   glShadeModel(GL_FLAT);

   alListenerfv(AL_POSITION, zeroes);
   alListenerfv(AL_VELOCITY, zeroes);
   alListenerfv(AL_ORIENTATION, front);

   alGetError();
   alGenBuffers(1, &boom);
   if(alGetError() != AL_NO_ERROR) {
      fprintf(stderr, "aldemo: couldn't generate samples\n");
      quit(1);
   }
   
#ifdef _LINUX
   alutLoadWAV("data/boom.wav", &wave, &format, &size, &bits, &freq);
#endif
#ifdef _WIN32
   alutLoadWAVFile("data/boom.wav", &format, &wave, &size, &freq, &loop);
#endif
   alBufferData(boom, format, wave, size, freq);

   alGetError();
   alGenSources(2, sources);

   if (alGetError() != AL_NO_ERROR) {
      fprintf(stderr, "aldemo: couldn't generate sources\n");
      quit(1);
   }

   left_sid = sources[0];
   right_sid = sources[1];

   alSourcefv(left_sid, AL_POSITION, left_pos);
   alSourcefv(left_sid, AL_VELOCITY, zeroes);
   alSourcefv(left_sid, AL_ORIENTATION, back);
   alSourcei(left_sid, AL_BUFFER, boom);

   alSourcefv(right_sid, AL_POSITION, right_pos);
   alSourcefv(right_sid, AL_VELOCITY, zeroes);
   alSourcefv(right_sid, AL_ORIENTATION, back);
   alSourcei(right_sid, AL_BUFFER, boom);
}

void mouse(int button, int state, int x, int y) 
{
   switch (button) {
      case SDL_BUTTON_LEFT:
         if (SDL_PRESSED == state) {
           spininc = 2.0;
	   alSourcePlay(left_sid);
         }
         break;
      case SDL_BUTTON_MIDDLE:
         if (SDL_PRESSED == state) {
           spininc = 0.0;
         }
         break;
      case SDL_BUTTON_RIGHT:
         if (SDL_PRESSED == state) {
           spininc = (-2.0);
	   alSourcePlay(right_sid);
         }
         break;
      default:
         break;
   }
}

/* 
 * Declare initial window size, position, and display mode
 * (single buffer and RGBA).  Open window with "hello"
 * in its title bar.  Call initialization routines.
 * Register callback function to display graphics.
 * Enter main loop and process events.
 */
int main(int argc, char** argv)
{
#ifdef _W32
   ALvoid *context;
   ALCdevice *device;
#endif

/* Information about the current video settings */
   const SDL_VideoInfo* info = NULL;
   SDL_Surface *screen = NULL;
/* SDL Event placeholder */
   SDL_Event event;
   
/* Dimensions of our window */
   int width = 0;
   int height = 0;
/* Color depth in bits of our window */
   int bpp = 0;
/* Flags we will pass into SDL_SetVideoMode */
   int flags = 0;

   printf("Initializing SDL.\n");

/* First, initialize SDL's video subsystem */
   if (0 > SDL_Init(SDL_INIT_VIDEO)) {
      fprintf(stderr, "Video initialization failed: %s\n", SDL_GetError());
      exit(EXIT_FAILURE);
   }

/* Let's get some video information */
   if (!(info = SDL_GetVideoInfo())) {
      fprintf(stderr, "Video query failed: %s\n", SDL_GetError());
      quit(EXIT_FAILURE);
   }

/*
 * Set our width/height to 640/480
 * and request the bpp from the system
 */
   width = 640;
   height = 480;
   bpp = info->vfmt->BitsPerPixel;

/*
 * Setup atleast 5 bits of red, green and blue.
 * Setup an depth buffer of atleast 16-bit.
 * Request double buffered window.
 */
   SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
   SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
   SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
   SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

/*
 * We request an SDL OpenGL window.
 * The SDL window should be Full Screen.
 */
 
   flags = (SDL_OPENGL | SDL_RESIZABLE);

/* Set the video mode */
   if (0 == (screen = SDL_SetVideoMode(width, height, bpp, flags))) {
       fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError());
       exit(EXIT_FAILURE);
   }
    
   printf("SDL initialized.\n");
#ifdef _WIN32
 	device=alcOpenDevice((ALubyte*)"DirectSound3D");
	context=alcCreateContext(device, NULL);
	alcMakeContextCurrent(context);
#else
    alutInit(&argc, argv);
#endif

   init();
   reshape(width, height);

   for (;;) {
      spinDisplay();
      while (SDL_PollEvent(&event)) {
         switch (event.type) {
         case SDL_VIDEORESIZE:
            screen = SDL_SetVideoMode(event.resize.w, event.resize.h, 0, (SDL_OPENGL | SDL_RESIZABLE));
	    if (screen) {
               reshape(screen->w, screen->h);
	    }
            break;
         case SDL_MOUSEBUTTONDOWN:
            mouse(event.button.button, event.button.state, event.button.x, event.button.y);
            break;
         case SDL_QUIT:
/* Handle quit requests (like Ctrl-c) */
            quit(EXIT_SUCCESS);
            break;
        }
     }
      display();
      SDL_Delay(50);
   }
     
   quit(EXIT_FAILURE);

   return 0;   /* ANSI C requires main to return int */
}
